/* 
 * Copyright (C) 2000  Florian Sander
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

static void incrstats(char *user, char *chan, int type, int value, int set)
{
  globstats *gs, *gs2;
  locstats *ls, *ls2;
  int i;

  Context;
  if (type >= TOTAL_TYPES)
    return;
  if (!user) {
    debug0("Stats.mod: incrstats(..) Ups, user is NULL!");
    return;
  }
  if (!chan) {
    debug0("Stats.mod: incrstats(..) Ups, chan is NULL!");
    return;
  }
  for (gs = sdata; gs; gs = gs->next) {
    if (!strcasecmp(chan, gs->chan))
      break;
  }
  if (!gs) {
    gs2 = sdata;
    while (gs2 && gs2->next)
      gs2 = gs2->next;
    gs = nmalloc(sizeof(globstats));
    gs->started = now;
    gs->peak[S_TOTAL] = 0;
    gs->peak[S_TODAY] = 0;
    gs->next = NULL;
    gs->local = NULL;
    gs->chan = nmalloc(strlen(chan) + 1);
    strcpy(gs->chan, chan);
    if (gs2)
      gs2->next = gs;
    else
      sdata = gs;
  }
  for (ls = gs->local; ls; ls = ls->next) {
    if (!strcasecmp(ls->user, user))
      break;
  }
  if (type == T_GSTARTED) {
    gs->started = value;
    return;
  }
  if (type == T_PEAK) {
    gs->peak[S_TOTAL] = value;
    return;
  }
  if (!ls) {
    ls2 = gs->local;
    while (ls2 && ls2->next)
      ls2 = ls2->next;
    ls = nmalloc(sizeof(locstats));
    ls->started = now;
    ls->next = NULL;
    for (i = 0; i < TOTAL_TYPES; i++) {
      ls->values[S_TOTAL][i] = 0;
      ls->values[S_TODAY][i] = 0;
    }
    ls->user = nmalloc(strlen(user) + 1);
    strcpy(ls->user, user);
    if (ls2)
      ls2->next = ls;
    else
      gs->local = ls;
  }
  if (type == T_LSTARTED)
    ls->started = value;
  else {
    if (set)
      ls->values[S_TOTAL][type] = value;
    else {
      ls->values[S_TOTAL][type] += value;
      ls->values[S_TODAY][type] += value;
    }
  }
}

static int getstats(char *user, char *chan, char *type, int today)
{
  struct stats_global *gs = sdata;
  struct stats_local *ls;
  
  Context;
  while (gs) {
    ls = gs->local;
    if (!strcasecmp(gs->chan, chan)) {
      while (ls) {
        if (!strcasecmp(ls->user, user)) {
          return ls->values[today][typetoi(type)];
        }
        ls = ls->next;
      }
    }
    gs = gs->next;
  }
  Context;
  return 0;
}

static void sortstats(struct stats_global *gs, char *type, int today)
{
  int again = 1;
  struct stats_local *last, *p, *c, *n;
  int a, b, itype;

  Context;
  again = 1;
  last = NULL;
  itype = typetoi(type);
  while ((gs->local != last) && (again)) {
    p = NULL;
    c = gs->local;
    n = c->next;
    again = 0;
    while (n != last) {
      if (!c || !n)
        a = b = 0;
      else if (!strcasecmp(type, "started")) {
        a = c->started;
        b = n->started;
      } else
        a = c->values[today][itype];
        b = n->values[today][itype];
      if (a < b) {
  again = 1;
  c->next = n->next;
  n->next = c;
  if (p == NULL)
    gs->local = n;
  else
    p->next = n;
      }
      p = c;
      c = n;
      n = n->next;
    }
    last = c;
  }
  Context;
  return;
}

static int typetoi(char *type)
{
  Context;
  if (!strcasecmp(type, "lstarted"))
    return T_LSTARTED;
  else if (!strcasecmp(type, "gstarted"))
    return T_GSTARTED;
  else if (!strcasecmp(type, "words"))
    return T_WORDS;
  else if (!strcasecmp(type, "letters"))
    return T_LETTERS;
  else if (!strcasecmp(type, "minutes"))
    return T_MINUTES;
  else if (!strcasecmp(type, "topics"))
    return T_TOPICS;
  else if (!strcasecmp(type, "lines"))
    return T_LINES;
  else if (!strcasecmp(type, "actions"))
    return T_ACTIONS;
  else if (!strcasecmp(type, "kicks"))
    return T_KICKS;
  else if (!strcasecmp(type, "modes"))
    return T_MODES;
  else if (!strcasecmp(type, "bans"))
    return T_BANS;
  else if (!strcasecmp(type, "nicks"))
    return T_NICKS;
  else if (!strcasecmp(type, "joins"))
    return T_JOINS;
  else if (!strcasecmp(type, "smileys"))
    return T_SMILEYS;
  else {
    debug1("Stats.mod: Unknown stat type: %s", type);
    return T_ERROR;
  }
}

static locstats *findlocstats(char *chan, char *user)
{
  globstats *gl;
  locstats *ll;
  
  Context;
  for (gl = sdata; gl; gl = gl->next) {
    if (!strcasecmp(gl->chan, chan))
      break;
  }
  if (!gl)
    return NULL;
  for (ll = gl->local; ll; ll = ll->next) {
    if (!strcasecmp(ll->user, user))
      return ll;
  }
  return NULL;
}

static globstats *findglobstats(char *chan)
{
  globstats *gl;
  
  Context;
  for (gl = sdata; gl; gl = gl->next) {
    if (!strcasecmp(gl->chan, chan))
      break;
  }
  return gl;
}

static void write_stats()
{
  char s[125];
  FILE *f;
  struct stats_global *gs;
  struct stats_local *ls;
  int i;

  Context;
  if (!statsfile[0])
    return;
  sprintf(s, "%s~new", statsfile);
  f = fopen(s, "w");
  chmod(s, statsfilemode);
  if (f == NULL) {
    putlog(LOG_MISC, "*", "ERROR writing stats file.");
    return;
  }
  fprintf(f, "@ Statistics from %s.\n", botnetnick);
  for (gs = sdata; gs; gs = gs->next) {
    fprintf(f, "%s ! %d\n", gs->chan, (int) gs->started);
    fprintf(f, "%s @ %d\n", gs->chan, gs->peak[S_TOTAL]);
    for (ls = gs->local; ls; ls = ls->next) {
      fprintf(f, "%s %s %d", gs->chan, ls->user, (int) ls->started);
      for (i = 0; i < TOTAL_TYPES; i++)
        fprintf(f, " %d", ls->values[S_TOTAL][i]);
      fprintf(f, "\n");
    }
  }
  fclose(f);
  unlink(statsfile);
  movefile(s, statsfile);
  Context;
  return;
}

static void read_stats()
{
  FILE *f;
  char buf[512];
  char *s, *chan, *user;
  int i;

  Context;
  f = fopen(statsfile, "r");
  if (f == NULL) {
    putlog(LOG_MISC, "*", "ERROR reading stats file");
    return;
  }
  free_stats();
  while (!feof(f)) {
    buf[0] = 0;
    s = buf;
    fgets(s, 511, f);
    if ((buf[0] == 0) || (buf[0] == '@'))
      continue;
    chan = newsplit(&s);
    user = newsplit(&s);
    if (!strcmp(user, "!"))
      incrstats(user, chan, T_GSTARTED, atoi(newsplit(&s)), 1);
    else if (!strcmp(user, "@"))
      incrstats(user, chan, T_PEAK, atoi(newsplit(&s)), 1);
    else {
      incrstats(user, chan, T_LSTARTED, atoi(newsplit(&s)), 1);
      for (i=0; i < TOTAL_TYPES; i++)
        incrstats(user, chan, i, atoi(newsplit(&s)), 1);
    }
  }
  fclose(f);
  Context;
  return;
}

static void reset_tstats()
{
  globstats *gs;
  locstats *ls;
  int i;

  Context;
  putlog(LOG_MISC, "*", "Stats.mod: Resetting today's statistics...");
  for (gs = sdata; gs; gs = gs->next) {
    gs->peak[S_TODAY] = 0;
    for (ls = gs->local; ls; ls = ls->next) {
      for (i = 0; i < TOTAL_TYPES; i++)
        ls->values[S_TODAY][i] = 0;
    }
  }
  Context;
}

static void sort_stats_alphabetically(globstats *gs)
{
  locstats *as, *bs, *l;
  int a, b, again = 1;
  
  while (again) {
    again = 0;
    l = NULL;
    as = gs->local;
    bs = gs->local->next;
    while(bs) {  
      if (!as)
        a = 0;
      else
        a = (int) as->user[0];
      if (!bs)
        b = 0;
      else
        b = (int) bs->user[0];
      if (a > b) {
        if (!l)
          gs->local = bs;
        else
          l->next = bs;
        as->next = bs->next;
        bs->next = as;
        again = 1;
        l = NULL; /* Improve me! */
        as = gs->local;
        bs = gs->local->next;
      } else {
        l = as;
        as = bs;
        bs = bs->next;
      }
    }
  }
}

static void resetlocstats(locstats *ls)
{
  int i;

  if (!ls) {
    debug0("ERROR! resetlocstats called with NULL pointer!");
    return;
  }
  for (i = 0; i < TOTAL_TYPES; i++) {
    ls->values[S_TOTAL][i] = 0;
    ls->values[S_TODAY][i] = 0;
  }
  return;
}