/* 
 * Copyright (C) 2000  Florian Sander
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

static int validchan(char *chan)
{
  struct chanset_t *ch;

#ifndef OLDBOT
  ch = findchan_by_dname(chan);
#else
  ch = findchan(chan);
#endif
  if (ch)
    return 1;
  return 0;
}

static int msg_top10 (char *nick, char *uhost, struct userrec *u, char *rest)
{
  char *chan;
  
  chan = newsplit(&rest);
  if (!validchan(chan)) {
    dprintf(DP_HELP, "PRIVMSG %s :Sorry, that is not a valid channel. Usage: top10 <chan>\n", nick);
    return 1;
  }
  putlog(LOG_CMDS, "*", "(%s!%s) !%s! top10 %s %s", nick, uhost, u->handle, chan, rest);
  tell_top(chan, nick, rest, 1, 10, 0);
  return 1;
}

static int msg_ttop10 (char *nick, char *uhost, struct userrec *u, char *rest)
{
  char *chan;
  
  chan = newsplit(&rest);
  if (!validchan(chan)) {
    dprintf(DP_HELP, "PRIVMSG %s :Sorry, that is not a valid channel. Usage: ttop10 <chan>\n", nick);
    return 1;
  }
  putlog(LOG_CMDS, "*", "(%s!%s) !%s! ttop10 %s %s", nick, uhost, u->handle, chan, rest);
  tell_top(chan, nick, rest, 1, 10, 1);
  return 1;
}

static int msg_top20 (char *nick, char *uhost, struct userrec *u, char *rest)
{
  char *chan;
  
  chan = newsplit(&rest);
  if (!validchan(chan)) {
    dprintf(DP_HELP, "PRIVMSG %s :Sorry, that is not a valid channel. Usage: top20 <chan>\n", nick);
    return 1;
  }
  putlog(LOG_CMDS, "*", "(%s!%s) !%s! top20 %s %s", nick, uhost, u->handle, chan, rest);
  tell_top(chan, nick, rest, 11, 20, 0);
  return 1;
}

static int msg_ttop20 (char *nick, char *uhost, struct userrec *u, char *rest)
{
  char *chan;
  
  chan = newsplit(&rest);
  if (!validchan(chan)) {
    dprintf(DP_HELP, "PRIVMSG %s :Sorry, that is not a valid channel. Usage: ttop10 <chan>\n", nick);
    return 1;
  }
  putlog(LOG_CMDS, "*", "(%s!%s) !%s! ttop20 %s %s", nick, uhost, u->handle, chan, rest);
  tell_top(chan, nick, rest, 11, 20, 1);
  return 1;
}

static int msg_place(char *nick, char *uhost, struct userrec *u,
        char *rest)
{
  char *chan;


  chan = newsplit(&rest);
  if (!validchan(chan)) {
    dprintf(DP_HELP, "PRIVMSG %s :Sorry, that is not a valid channel. Usage: place <chan>\n", nick);
    return 1;
  }
  putlog(LOG_CMDS, "*", "(%s!%s) !%s! place %s %s", nick, uhost, u->handle, chan, rest);
  tell_place(nick, nick, u->handle, chan, rest, S_TOTAL);
  Context;
  return 1;
}

static int msg_tplace(char *nick, char *uhost, struct userrec *u,
        char *rest)
{
  char *chan;

  chan = newsplit(&rest);
  if (!validchan(chan)) {
    dprintf(DP_HELP, "PRIVMSG %s :Sorry, that is not a valid channel. Usage: tplace <chan>\n", nick);
    return 1;
  }
  putlog(LOG_CMDS, "*", "(%s!%s) !%s! tplace %s %s", nick, uhost, u->handle, chan, rest);
  tell_place(nick, nick, u->handle, chan, rest, S_TODAY);
  Context;
  return 1;
}

static int msg_stat(char *nick, char *uhost, struct userrec *u,
        char *rest)
{
  char *chan;

  chan = newsplit(&rest);
  if (!validchan(chan)) {
    dprintf(DP_HELP, "PRIVMSG %s :Sorry, that is not a valid channel. Usage: stat <chan> [user]\n", nick);
    return 1;
  }
  putlog(LOG_CMDS, "*", "(%s!%s) !%s! stat %s %s", nick, uhost, u->handle, chan, rest);
  tell_stat(nick, nick, u->handle, chan, rest, S_TOTAL);
  Context;
  return 1;
}

static int msg_tstat(char *nick, char *uhost, struct userrec *u,
        char *rest)
{
  char *chan;

  chan = newsplit(&rest);
  if (!validchan(chan)) {
    dprintf(DP_HELP, "PRIVMSG %s :Sorry, that is not a valid channel. Usage: tstat <chan> [user]\n", nick);
    return 1;
  }
  putlog(LOG_CMDS, "*", "(%s!%s) !%s! tstat %s %s", nick, uhost, u->handle, chan, rest);
  tell_stat(nick, nick, u->handle, chan, rest, S_TODAY);
  Context;
  return 1;
}

static cmd_t stats_msg[] =
{
  {"top10", "", msg_top10, 0},
  {"ttop10", "", msg_ttop10, 0},
  {"top20", "", msg_top20, 0},
  {"ttop20", "", msg_ttop20, 0},
  {"place", "", msg_place, 0},
  {"tplace", "", msg_tplace, 0},
  {"stat", "", msg_stat, 0},
  {"tstat", "", msg_tstat, 0},
  {0, 0, 0, 0}
};
